#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# import autostream
import threading
import time
import os
import sys
import traceback


class ManageBot():
    """
    AutoTrader Bot Manager
    ----------------------
    Detaches from AutoTrader run script to allow for a single deployment.
    
    Attributes
    ----------
    bot: class
        The bot being managed.
    
    Methods 
    --------
    update_bot_data():
        Passes the latest price data to the bot.
    
    kill_bot():
        Terminates the bot from trading.
    
    write_bot_to_log():
        Adds the bot being managed to the bots_deployed logfile.
    
    remove_bot_from_log():
        Removes the bot being managed from the bots_deployed logfile.
    
    
    Strategies being deployed to bot manager must have the following methods:
        - initialise_strategy()
        - exit_strategy(i)
    
    As well as a "terminate" attribute.
    
    Currently, there is only one way to intervene and kill a bot. This is to
    create an empty file named 'killbots' in the home_dir. Note that this
    will kill all bots running. In future updates, killing selected bots will 
    be supported. Options for this include:
        - creating an empty file with name related to specific bot
        - bot manager will create an empty file corresponding to each bot 
          (perhaps in a bots_deployed directory), allowing killing of bots by
          deleting their specific file.
    
    """
    
    def __init__(self, bot, home_dir, bot_name_string, use_stream):
        
        self.bot = bot
        self.home_dir = home_dir
        self.managing = True
        self.use_stream = use_stream
        
        self.active_bots_dir = os.path.join(home_dir, 'active_bots')
        self.active_bot_path = os.path.join(self.active_bots_dir, bot_name_string)
        self.killfile = os.path.join(self.home_dir, 'killbots')
        self.suspendfile = os.path.join(self.home_dir, 'suspendbots')
        
        # Create name string
        self.bot_name_string = bot_name_string
        
        # Check if active_bots directory exists
        if not os.path.isdir(self.active_bots_dir):
            # Directory does not exist, create it
            os.mkdir(self.active_bots_dir)
        
        # Check if bot is already deployed
        bot_already_deployed = self.check_bots_deployed()
        
        if not bot_already_deployed:
            # Spawn new thread for bot manager
            thread = threading.Thread(target=self.manage_bot, args=(), 
                                      daemon=False)
            print("Bot recieved. Now managing bot '{}'.".format(bot_name_string))
            print("To kill bot, delete from bots_deployed directory.")
            print("Alternatively create file named 'killbots' in the home_dir" \
                  + " to kill all bots.\n")
            print("You can also suspend a bot by creating a 'suspendbots' file." \
                  + " Deleting this file will then resume the bots.")
            thread.start()
        else:
            print("Notice: Bot has already been deployed. Exiting.")
        
    def manage_bot(self):
        '''
        Manages bot until terminal condition is met.
        '''
        
        # Add bot to log
        self.write_bot_to_log()
        
        # Signal that bot is ready to recieve data from stream
        if self.use_stream:
            self.bot._recieve_stream_data()
        
        # Manage
        while self.managing:
            
            # First check for any termination signals
            if self.bot.strategy.terminate:
                print("\nBot will be terminated.")
                self.remove_bot_from_log()
                
                # End management
                self.managing = False
            
            elif not os.path.exists(self.active_bot_path):
                print("\nBot file deleted. Bot will be terminated.")
                
                if self.use_stream:
                    # Sleep for 5 seconds to allow for any residual stream actions 
                    time.sleep(5)
                    
                self.bot.strategy.exit_strategy(-1)
                
                # End management
                self.managing = False
            
            elif os.path.exists(self.killfile):
                print("\nKillfile detected. Bot will be terminated.")
                
                if self.use_stream:
                    # Sleep for 5 seconds to allow for any residual stream actions 
                    time.sleep(5)
                    
                self.bot.strategy.exit_strategy(-1)
                
                # Remove bot from log
                self.remove_bot_from_log()
                
                # End management
                self.managing = False
            
            elif os.path.exists(self.suspendfile):
                print("\nSuspending {} bot.".format(self.bot_name_string))
                self.suspend()
                print("Resuming {} bot.".format(self.bot_name_string))
                
            else:
                # No termination signal detected, proceed to manage
                
                if not self.use_stream:
                    # Periodic update mode
                    for atempt in range(3):
                        try:
                            # Refresh strategy with latest data
                            self.bot._update_strategy_data()
                            
                            # Call bot update to act on latest data
                            self.bot._update(-1)
                        
                        except BaseException as ex:
                            # Get current system exception
                            ex_type, ex_value, ex_traceback = sys.exc_info()
                        
                            # Extract unformatter stack traces as tuples
                            trace_back = traceback.extract_tb(ex_traceback)
                        
                            # Format stacktrace
                            stack_trace = list()
                        
                            for trace in trace_back:
                                trade_string = "File : %s , Line : %d, " % (trace[0], trace[1]) + \
                                               "Func.Name : %s, Message : %s" % (trace[2], trace[3])
                                stack_trace.append(trade_string)
                            
                            print("WARNING FROM BOT MANAGER: The following exception was caught " +\
                                  "when updating {}.".format(self.bot_name_string))
                            print("Exception type : %s " % ex_type.__name__)
                            print("Exception message : %s" %ex_value)
                            print("Stack trace : %s" %stack_trace)
                            print("  Trying again.")
                        
                        else:
                            break
                        
                    else:
                        print("FATAL: All attempts have failed. Going to sleep.")
                    
                    # Pause an amount, depending on granularity
                    base_granularity = self.bot.strategy_params['granularity'].split(',')[0]
                    if base_granularity == 'tick':
                        time.sleep(3)
                    else:
                        sleep_time = 0.25*self.granularity_to_seconds(base_granularity)
                        time.sleep(sleep_time)

    def suspend(self):
        while os.path.exists(self.suspendfile):
            pass    
            
    def write_bot_to_log(self):
        '''
        Adds the bot being managed to the bots_deployed logfile.
        '''
        
        with open(self.active_bot_path, 'w') as f:
            pass
    
    def remove_bot_from_log(self):
        '''
        Removes the bot being managed from the bots_deployed logfile.
        '''
        
        os.remove(self.active_bot_path)
        
    
    def check_bots_deployed(self):
        '''
        Checks the bots currently deployed to prevent a re-deployment.
        '''
        
        if os.path.exists(self.active_bot_path):
            return True
        else:
            return False
        

    def granularity_to_seconds(self, granularity):
        '''Converts the interval to time in seconds'''
        letter = granularity[0]
        
        if len(granularity) > 1:
            number = float(granularity[1:])
        else:
            number = 1
        
        conversions = {'S': 1,
                       'M': 60,
                       'H': 60*60,
                       'D': 60*60*24
                       }
        
        seconds = conversions[letter] * number
        
        return seconds



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Function to parse scan request and return watchlist.

"""
import sys

def get_watchlist(index, feed):
    '''
    This code is currently in development.
    
    Returns a watchlist of instruments. 
    
    Objectives: 
        - return specific list of instruments based on input
          For example; 
              - forex:major -> major forex pairs
              - stocks:asx200
              - custom
    
    The current implementation only support forex indices, with Oanda 
    formatting.
    
    '''
    
    if len(index) == 0:
        print("\nArgument for scan missing. Please specify instrument/index to scan.")
        print("Try $ ./AutoTrader.py -h s for more help.\n")
        sys.exit(0)
    
    if index == 'all':
        ''' Returns all currency pairs. '''
        watchlist = ['EUR_USD', 
                     'USD_JPY', 
                     'GBP_USD', 
                     'AUD_USD', 
                     'USD_CAD', 
                     'USD_CHF', 
                     'NZD_USD',
                     'EUR_GBP',
                     'EUR_AUD',
                     'EUR_CAD',
                     'EUR_CHF',
                     'EUR_JPY',
                     'EUR_NZD',
                     'GBP_JPY',
                     'GBP_AUD',
                     'GBP_CAD',
                     'GBP_CHF',
                     'GBP_NZD',
                     'AUD_CAD',
                     'AUD_CHF',
                     'AUD_JPY',
                     'AUD_NZD',
                     'CAD_CHF',
                     'CAD_JPY',
                     'CHF_JPY',
                     'NZD_CHF',
                     'NZD_JPY',
                     ]
    
    elif index == 'major':
        ''' Returns major currency pairs. '''
        if feed.lower() == 'oanda':
            watchlist = ['EUR_USD', 
                         'USD_JPY', 
                         'GBP_USD', 
                         'AUD_USD', 
                         'USD_CAD', 
                         'USD_CHF', 
                         'NZD_USD'
                         ]
            
        elif feed.lower() == 'yahoo':
            watchlist = ['EURUSD=X', 
                         'USDJPY=X', 
                         'GBPUSD=X', 
                         'AUDUSD=X', 
                         'USDCAD=X', 
                         'USDCHF=X', 
                         'NZDUSD=X'
                         ]
        
    elif index == 'minor':
        ''' Returns minor currency pairs. '''
        
        if feed.lower() == 'oanda':
            watchlist = ['EUR_GBP',
                         'EUR_AUD',
                         'EUR_CAD',
                         'EUR_CHF',
                         'EUR_JPY',
                         'EUR_NZD',
                         'GBP_JPY',
                         'GBP_AUD',
                         'GBP_CAD',
                         'GBP_CHF',
                         'GBP_NZD',
                         'AUD_CAD',
                         'AUD_CHF',
                         'AUD_JPY',
                         'AUD_NZD',
                         'CAD_CHF',
                         'CAD_JPY',
                         'CHF_JPY',
                         'NZD_CHF',
                         'NZD_JPY']
            
        elif feed.lower() == 'yahoo':
            watchlist = ['EURGBP=X',
                         'EURAUD=X',
                         'EURCAD=X',
                         'EURCHF=X',
                         'EURJPY=X',
                         'EURNZD=X',
                         'GBPJPY=X',
                         'GBPAUD=X',
                         'GBPCAD=X',
                         'GBPCHF=X',
                         'GBPNZD=X',
                         'AUDCAD=X',
                         'AUDCHF=X',
                         'AUDJPY=X',
                         'AUDNZD=X',
                         'CADCHF=X',
                         'CADJPY=X',
                         'CHFJPY=X',
                         'NZDCHF=X',
                         'NZDJPY=X']
    
    elif index == 'exotic':
        ''' Returns exotic currency pairs. '''
        watchlist = ['EUR_TRY',
        	      'USD_HKD',
        	      'JPY_NOK',
        	      'NZD_SGD',
        	      'GBP_ZAR',
        	      'AUD_MXN']
    
    
    elif index[3] == "_":
        watchlist = [index]
    
    else:
        print("Not supported.")
        sys.exit(0)

    
    return watchlist





import pandas as pd
import numpy as np
import os

# Bokeh
from bokeh.models.annotations import Title
from bokeh.plotting import figure, output_file, show, save
from bokeh.models import (
    CustomJS,
    ColumnDataSource,
    HoverTool,
    CrosshairTool,
    Span
)
from bokeh.layouts import gridplot, layout
from bokeh.transform import factor_cmap, cumsum
from bokeh.palettes import Category20c
from math import pi


class AutoPlot():
    '''
    Attributes
    ----------
    data : df
        The base data used to plot candles. Required upon instantiation.
    
    max_inis_over : int
        The maximum number of indicators to overlay onto the candlestick plot.
    
    max_indis_below : int
        The maximum number of indicators to plot on new figures below the main
        candlestick plot.
    
    fig_tools : str
        A comma separated string of Bokeh plotting tools.
        
    ohlc_height : int
        The height of the candlestick plot in pixels.
    
    ohlc_width : int
        The width of the candlestick plot in pixels.
        
    top_fig_height : int
        The height of the figure above the candlestick plot in pixels.
    
    bottom_fig_height : int
        The height of the figures below the candlestick plot in pixels.


    Methods
    -------
    __init__(data):
        Initialise AutoPlot with the lowest timeframe data being used for 
        plotting.
    
    add_tool(tool_name):
        Add bokeh tool to plot. This adds the tool_name string to the fig_tools
        attribute.
    
    plot(backtest_dict=None, cumulative_PL=None, indicators=None, instrument=None, show_fig=True):
        Creates a plot.
    
    '''
    
    def __init__(self, data):
        self.max_indis_over     = 3
        self.max_indis_below    = 2
        self._modified_data     = None
        self.fig_tools          = "pan,wheel_zoom,box_zoom,undo,redo,reset,save,crosshair"
        self.ohlc_height        = 400
        self.ohlc_width         = 800
        self.top_fig_height     = 150
        self.bottom_fig_height  = 150
        # self.total_height       = 1000
        
        # Modify data index
        self.data               = self._reindex_data(data)
        
        # Load JavaScript code for auto-scaling 
        with open(os.path.join(os.path.dirname(__file__), 'lib/autoscale.js'),
                  encoding = 'utf-8') as _f:
            self._autoscale_code = _f.read()
        
        
    def add_tool(self, tool_name):
        '''
        Adds a tool to the plot. 
        
            Parameters:
                tool_name (str): name of tool to add (see Bokeh documentation).
        '''
        
        self.fig_tools          = self.fig_tools + "," + tool_name
    
    
    def _reindex_data(self, data):
        '''
        Resets index of data to obtain integer indexing.
        '''
        
        modified_data           = data.copy()
        modified_data['date']   = modified_data.index
        modified_data           = modified_data.reset_index(drop = True)
        modified_data['data_index'] = modified_data.index
        
        return modified_data
    
    
    ''' ------------------- FIGURE MANAGEMENT METHODS --------------------- '''
    def plot(self, backtest_dict=None, cumulative_PL=None, indicators=None, 
             instrument=None, show_fig=True):
        ''' 
        Creates chart of price data and indicators. 
        '''
        
        # Preparation ------------------------------------------------------- #
        if backtest_dict is None:
            if instrument is not None:
                title_string = "AutoTrader IndiView - {}".format(instrument)
            else:
                title_string = "AutoTrader IndiView"
            
            output_file("indiview-chart.html",
                        title = title_string)
            
        else:
            if instrument is None:
                instrument = backtest_dict['instrument']
            
            title_string = "Backtest chart for {} ({} candles)".format(instrument, 
                                                                       backtest_dict['interval'])
            output_file("{}-backtest-chart.html".format(instrument),
                        title = "AutoTrader Backtest Results - {}".format(instrument))
        
        source = ColumnDataSource(self.data)
        source.add((self.data.Close >= self.data.Open).values.astype(np.uint8).astype(str),
                   'change')
        
        
        # Plotting ---------------------------------------------------------- #
        # OHLC candlestick plot
        candle_plot = self._plot_candles(source)
        
        top_figs = []
        bottom_figs = []
        
        if backtest_dict is not None:
            
            NAV             = backtest_dict['NAV']
            balance         = backtest_dict['balance']
            trade_summary   = backtest_dict['trade_summary']
            indicators      = backtest_dict['indicators']
            open_trades     = backtest_dict['open_trades']
            cancelled_trades = backtest_dict['cancelled_trades']
            
            # Top plots
            top_fig = self._plot_line(NAV, candle_plot, new_fig=True, 
                                      legend_label='Net Asset Value', 
                                      hover_name='NAV')
            self._plot_line(balance, 
                            top_fig, 
                            legend_label='Account Balance', 
                            hover_name='P/L',
                            line_colour='blue')
            # if cumulative_PL is not None:
                
            #     self._plot_line(cumulative_PL, top_fig, 
            #                     legend_label='Cumulative P/L', 
            #                     hover_name='P/L')
            top_figs.append(top_fig)
            
            # Overlay trades 
            self._plot_trade_history(trade_summary, candle_plot)
            if len(cancelled_trades) > 0:
                self._plot_trade_history(cancelled_trades, candle_plot, cancelled_summary=True)
            if len(open_trades) > 0:
                self._plot_trade_history(open_trades, candle_plot, open_summary=True)
        
        # Indicators
        if indicators is not None:
            bottom_figs = self._plot_indicators(indicators, candle_plot)
        
        
        # Compile plots for final figure ------------------------------------ #
        # Auto-scale y-axis of candlestick chart - TODO - improve
        autoscale_args      = dict(y_range  = candle_plot.y_range, 
                                   source   = source)
        candle_plot.x_range.js_on_change('end', CustomJS(args = autoscale_args, 
                                                         code = self._autoscale_code))
        
        plots               = top_figs + [candle_plot] + bottom_figs
        linked_crosshair    = CrosshairTool(dimensions='both')
        
        titled  = 0
        t       = Title()
        t.text  = title_string
        for plot in plots:
            if plot is not None:
                plot.xaxis.major_label_overrides = {
                    i: date.strftime('%b %d') for i, date in enumerate(pd.to_datetime(self.data["date"]))
                }
                plot.xaxis.bounds   = (0, self.data.index[-1])
                plot.sizing_mode    = 'stretch_width'
                
                if titled == 0:
                    plot.title = t
                    titled = 1
                
                if plot.legend:
                    plot.legend.visible             = True
                    plot.legend.location            = 'top_left'
                    plot.legend.border_line_width   = 1
                    plot.legend.border_line_color   = '#333333'
                    plot.legend.padding             = 5
                    plot.legend.spacing             = 0
                    plot.legend.margin              = 0
                    plot.legend.label_text_font_size = '8pt'
                    plot.legend.click_policy        = "hide"
                
                plot.add_tools(linked_crosshair)
                plot.min_border_left    = 0
                plot.min_border_top     = 3
                plot.min_border_bottom  = 6
                plot.min_border_right   = 10
                plot.outline_line_color = 'black'
    
        # Construct final figure
        fig                 = gridplot(plots, 
                                       ncols            = 1, 
                                       toolbar_location = 'right',
                                       toolbar_options  = dict(logo = None), 
                                       merge_tools      = True
                                       )
        fig.sizing_mode     = 'stretch_width'
        
        if show_fig:
            show(fig)
        else:
            save(fig)
    
    
    def _plot_multibot_backtest(self, multibot_backtest_results, NAV, cpl_dict):
        ''' 
        Creates multi-bot backtest figure. 
        
            Parameters:
                multibot_backtest_results (df): dataframe of bot backtest results.
                
                NAV (list): Net asset value.
                
                cpl_dict (dict): cumulative PL of each bot.
        '''
        
        # TODO - merge this into self.plot method?
        # First, clean up individual plots (pie, etc) into new methods
        
        # Preparation ----------------------------------- #
        output_file("candlestick.html",
                    title = "AutoTrader Multi-Bot Backtest Results")
        
        linked_crosshair = CrosshairTool(dimensions='both')
        if len(multibot_backtest_results) < 3:
            multibot_backtest_results['color'] = Category20c[3][0:len(multibot_backtest_results)]
        else:
            multibot_backtest_results['color'] = Category20c[len(multibot_backtest_results)]
            
        MBR = ColumnDataSource(multibot_backtest_results)
        
        # ----------------------- Account Balance -------------------------- #
        navfig = figure(plot_width = self.ohlc_width,
                        plot_height = self.top_fig_height,
                        title = None,
                        active_drag = 'pan',
                        active_scroll = 'wheel_zoom')
        
        # Add glyphs
        navfig.line(self.data.index, 
                    NAV, 
                    line_color = 'black',
                    legend_label = 'Backtest Net Asset Value')
        
        navfig.xaxis.major_label_overrides = {
                    i: date.strftime('%b %d') for i, date in enumerate(pd.to_datetime(self.data["date"]))
                }
        navfig.xaxis.bounds = (0, self.data.index[-1])
        navfig.sizing_mode = 'stretch_width'
        navfig.legend.location = 'top_left'
        navfig.legend.border_line_width   = 1
        navfig.legend.border_line_color   = '#333333'
        navfig.legend.padding             = 5
        navfig.legend.spacing             = 0
        navfig.legend.margin              = 0
        navfig.legend.label_text_font_size = '8pt'
        navfig.add_tools(linked_crosshair)
        
        # ----------------------- Win rate bar chart ----------------------- #
        instruments = multibot_backtest_results.index.values
        
        winrate = self._plot_bars(instruments, 'win_rate', MBR, 
                                  fig_title='Bot win rate (%)',
                                  hover_name='win_rate%')
        
        winrate.sizing_mode = 'stretch_width'
        
        
        # ----------------- Pie chart of trades per bot --------------------- #
        pie_data = pd.Series(multibot_backtest_results.no_trades).reset_index(name='value').rename(columns={'index':'instrument'})
        pie_data['angle'] = pie_data['value']/pie_data['value'].sum() * 2*pi
        if len(multibot_backtest_results) < 3:
            pie_data['color'] = Category20c[3][0:len(multibot_backtest_results)]
        else:
            pie_data['color'] = Category20c[len(multibot_backtest_results)]

        pie = self._plot_pie(pie_data, fig_title="Trade distribution")
        
        pie.axis.axis_label=None
        pie.axis.visible=False
        pie.grid.grid_line_color = None
        pie.sizing_mode = 'stretch_width'
        pie.legend.location = "top_left"
        pie.legend.border_line_width   = 1
        pie.legend.border_line_color   = '#333333'
        pie.legend.padding             = 5
        pie.legend.spacing             = 0
        pie.legend.margin              = 0
        pie.legend.label_text_font_size = '8pt'
        
        # --------------- Bar plot for avg/max win/loss -------------------- #
        win_metrics = ['Average Win', 'Max. Win']
        lose_metrics = ['Average Loss', 'Max. Loss']
        
        abs_max_loss = -1.2*max(multibot_backtest_results.max_loss)
        abs_max_win = 1.2*max(multibot_backtest_results.max_win)
        
        pldata = {'instruments': instruments,
                'Average Win': multibot_backtest_results.avg_win.values,
                'Max. Win': multibot_backtest_results.max_win.values - 
                            multibot_backtest_results.avg_win.values,
                'Average Loss': -multibot_backtest_results.avg_loss.values,
                'Max. Loss': multibot_backtest_results.avg_loss.values - 
                             multibot_backtest_results.max_loss.values}
        
        TOOLTIPS = [
                    ("Instrument:", "@instruments"),
                    ("Max win", "@{Max. Win}"),
                    ("Avg. win", "@{Average Win}"),
                    ("Max Loss", "@{Max. Loss}"),
                    ("Avg. loss", "@{Average Loss}"),
                    ]
        
        plbars = figure(x_range=instruments,
                        y_range=(abs_max_loss, abs_max_win),
                        title="Win/Loss breakdown",
                        toolbar_location=None,
                        tools = "hover",
                        tooltips = TOOLTIPS,
                        plot_height = 250)

        plbars.vbar_stack(win_metrics,
                     x='instruments',
                     width = 0.9,
                     color = ('#008000', '#FFFFFF'),
                     line_color='black',
                     source = ColumnDataSource(pldata),
                     legend_label = ["%s" % x for x in win_metrics])

        plbars.vbar_stack(lose_metrics,
                     x = 'instruments',
                     width = 0.9,
                     color = ('#ff0000' , '#FFFFFF'),
                     line_color='black',
                     source = ColumnDataSource(pldata),
                     legend_label = ["%s" % x for x in lose_metrics])
        
        plbars.x_range.range_padding = 0.1
        plbars.ygrid.grid_line_color = None
        plbars.legend.location = "bottom_center"
        plbars.legend.border_line_width   = 1
        plbars.legend.border_line_color   = '#333333'
        plbars.legend.padding             = 5
        plbars.legend.spacing             = 0
        plbars.legend.margin              = 0
        plbars.legend.label_text_font_size = '8pt'
        plbars.axis.minor_tick_line_color = None
        plbars.outline_line_color = None
        plbars.sizing_mode = 'stretch_width'
    
    
        # --------------------- Cumulative PL ------------------------------ #
        cplfig = figure(plot_width = navfig.plot_width,
                        plot_height = self.top_fig_height,
                        title = None,
                        active_drag = 'pan',
                        active_scroll = 'wheel_zoom',
                        x_range = navfig.x_range)
        
        # self.data['data_index'] = self.data.reset_index(drop=True).index
        
        if len(multibot_backtest_results) < 3:
            colors = Category20c[3][0:len(multibot_backtest_results)]
        else:
            colors = Category20c[len(multibot_backtest_results)]
        
        
        for ix, instrument in enumerate(cpl_dict):
            cpldata = cpl_dict[instrument].copy().to_frame()
            cpldata['date'] = cpldata.index
            cpldata         = cpldata.reset_index(drop = True)
            
            cpldata = pd.merge(self.data, cpldata, left_on='date', right_on='date')
            
            cplfig.line(cpldata.data_index.values,
                        cpldata.Profit.values,
                        legend_label = "{}".format(instrument),
                        line_color = colors[ix])
        
        cplfig.legend.location = 'top_left'
        cplfig.legend.border_line_width   = 1
        cplfig.legend.border_line_color   = '#333333'
        cplfig.legend.padding             = 5
        cplfig.legend.spacing             = 0
        cplfig.legend.margin              = 0
        cplfig.legend.label_text_font_size = '8pt'
        cplfig.sizing_mode = 'stretch_width'
        cplfig.add_tools(linked_crosshair)
        
        cplfig.xaxis.major_label_overrides = {
                    i: date.strftime('%b %d') for i, date in enumerate(pd.to_datetime(self.data["date"]))
                }
        cplfig.xaxis.bounds   = (0, self.data.index[-1])
        
        # -------------------- Construct final figure ---------------------- #     
        final_fig = layout([  
                                   [navfig],
                            [winrate, pie, plbars],
                                   [cplfig]
                        ])
        final_fig.sizing_mode = 'scale_width'
        show(final_fig)
    
        
    def _plot_indicators(self, indicators, linked_fig):
        ''' 
        Plots indicators based on indicator type. If inidcator type is 
        "over", it will be plotted on top of linked_fig. If indicator type is 
        "below", it will be plotted on a new figure below the OHLC chart.
        '''
        
        x_range   = self.data.index
        
        plot_type = {'MACD'        : 'below',
                     'MA'          : 'over',
                     'RSI'         : 'below',
                     'STOCHASTIC'  : 'below',
                     'Heikin-Ashi' : 'below',
                     'Supertrend'  : 'over',
                     'Swings'      : 'over',
                     'Engulfing'   : 'below',
                     'Crossover'   : 'below',
                     'over'        : 'over',
                     'below'       : 'below',
                     'Grid'        : 'over'}
        
        # Plot indicators
        indis_over              = 0
        indis_below             = 0
        bottom_figs             = []
        colours                 = ['red', 'blue', 'orange', 'green']
        
        for indicator in indicators:
            indi_type = indicators[indicator]['type']
            
            if indi_type in plot_type:
                # The indicator plot type is recognised
                if plot_type[indi_type] == 'over' and indis_over < self.max_indis_over:
                    if indi_type == 'Supertrend':
                        self._plot_supertrend(indicators[indicator]['data'], 
                                              linked_fig)
                        
                        indis_over     += 1 # Count as 2 indicators
                    elif indi_type == 'Swings':
                        self._plot_swings(indicators[indicator]['data'], 
                                          linked_fig)
                    elif indi_type == 'Grid':
                        self._plot_grid(indicators[indicator]['data'], 
                                        linked_fig)
                    else:
                        # Generic overlay indicator - plot as line
                        linked_fig.line(x_range, 
                                        indicators[indicator]['data'], 
                                        line_width = 1.5, 
                                        legend_label = indicator,
                                        line_color = colours[indis_over])
                    indis_over     += 1
                    
                elif plot_type[indi_type] == 'below' and indis_below < self.max_indis_below:
                    if indi_type == 'MACD':
                        new_fig     = self._plot_macd(x_range,
                                                      indicators[indicator], 
                                                      linked_fig)
                        new_fig.title = indicator
                    
                    elif indi_type == 'Heikin-Ashi':
                        
                        HA_data     = self._reindex_data(indicators[indicator]['data'])
                        source      = ColumnDataSource(HA_data)
                        source.add((HA_data.Close >= HA_data.Open).values.astype(np.uint8).astype(str),
                                   'change')
                        new_fig     = self._plot_candles(source)
                        new_fig.x_range = linked_fig.x_range
                        new_fig.y_range = linked_fig.y_range
                        new_fig.title = indicator
                        indis_below   += self.max_indis_below # To block any other new plots below.
                    
                    else:
                        new_fig = figure(plot_width     = linked_fig.plot_width,
                                         plot_height    = 130,
                                         title          = None,
                                         tools          = linked_fig.tools,
                                         active_drag    = linked_fig.tools[0],
                                         active_scroll  = linked_fig.tools[1],
                                         x_range        = linked_fig.x_range)
                        
                        # Add glyphs
                        new_fig.line(x_range, 
                                     indicators[indicator]['data'],
                                     line_color         = 'black', 
                                     legend_label       = indicator)
                        
                    indis_below    += 1
                    bottom_figs.append(new_fig)
            else:
                # The indicator plot type is not recognised - plotting on new fig
                if indis_below < self.max_indis_below:
                    print("Indicator type '{}' not recognised in AutoPlot.".format(indi_type))
                    new_fig = self._plot_line(indicators[indicator]['data'], 
                                              linked_fig, new_fig=True, 
                                              legend_label=indicator, 
                                              hover_name=indicator)
                    
                    indis_below    += 1
                    bottom_figs.append(new_fig)
                
        return bottom_figs
    
    
    def _plot_line(self, plot_data, linked_fig, new_fig=False, fig_height=150,
                   fig_title=None, legend_label=None, hover_name=None,
                   line_colour='black'):
        '''
        Generic method to plot data as a line.
        '''
        
        # Initiate figure
        if new_fig:
            fig = figure(plot_width     = linked_fig.plot_width,
                         plot_height    = fig_height,
                         title          = fig_title,
                         tools          = self.fig_tools,
                         active_drag    = 'pan',
                         active_scroll  = 'wheel_zoom',
                         x_range        = linked_fig.x_range)
        else:
            fig = linked_fig
        
        # Add glyphs
        source = ColumnDataSource(self.data)
        source.add(plot_data, 'plot_data')
        fig.line('data_index', 'plot_data', 
                 line_color         = line_colour,
                 legend_label       = legend_label,
                 source             = source)
        
        if hover_name is None:
            hover_name = 'Data'
        
        fig_hovertool = HoverTool(tooltips = [("Date", "@date{%b %d %H:%M}"),
                                              (hover_name, "$@{plot_data}{%0.2f}")
                                              ], 
                                  formatters={'@{plot_data}' : 'printf',
                                              '@date' : 'datetime'},
                                  mode = 'vline')
        
        fig.add_tools(fig_hovertool)
        
        return fig
    
    
    ''' ------------------------ OVERLAY PLOTTING ------------------------- '''
    
    def _plot_candles(self, source):
        ''' Plots OHLC data onto new figure. '''
        bull_colour             = "#D5E1DD"
        bear_colour             = "#F2583E"
        candle_colours          = [bear_colour, bull_colour]
        colour_map              = factor_cmap('change', candle_colours, ['0', '1'])
        
        candle_tooltips         = [("Date", "@date{%b %d %H:%M:%S}"),
                                   ("Open", "@Open{0.0000}"), 
                                   ("High", "@High{0.0000}"), 
                                   ("Low", "@Low{0.0000}"),
                                   ("Close", "@Close{0.0000}")]
    
        candle_plot = figure(plot_width     = self.ohlc_width, 
                             plot_height    = self.ohlc_height, 
                             tools          = self.fig_tools,
                             active_drag    = 'pan',
                             active_scroll  = 'wheel_zoom')
    
        candle_plot.segment('index', 'High',
                            'index', 'Low', 
                            color   = "black",
                            source  = source)
        candles = candle_plot.vbar('index', 0.7, 'Open', 'Close', 
                                   source       = source,
                                   line_color   = "black", 
                                   fill_color   = colour_map)
        
        candle_hovertool = HoverTool(tooltips   = candle_tooltips, 
                                  formatters    = {'@date':'datetime'}, 
                                  mode          = 'vline',
                                  renderers     = [candles])
        
        candle_plot.add_tools(candle_hovertool)
        
        return candle_plot
    
    def _plot_swings(self, swings, linked_fig):
        '''
        Plots swing detection indicator.
        '''
        swings = pd.merge(self.data, swings, left_on='date', right_index=True)
        
        linked_fig.scatter(list(swings.index),
                            list(swings.Last.values),
                            marker = 'dash',
                            size = 15,
                            fill_color = 'black',
                            legend_label = 'Last Swing Price Level')
    
    def _plot_supertrend(self, st_data, linked_fig):
        ''' Plots supertrend indicator. '''
        # Extract supertrend data
        # uptrend     = st_data['uptrend']
        # dntrend     = st_data['downtrend']
        
        # reset index 
        st_data['date']         = st_data.index 
        st_data                 = st_data.reset_index(drop = True)
        
        # Add glyphs
        linked_fig.scatter(st_data.index,
                    st_data['uptrend'],
                    size = 5,
                    fill_color = 'blue',
                    legend_label = 'Up trend support')
        linked_fig.scatter(st_data.index,
                    st_data['downtrend'],
                    size = 5,
                    fill_color = 'red',
                    legend_label = 'Down trend support')
    
    def _plot_grid(self, grid_levels, linked_fig, linewidth=0.5):
        for price in grid_levels:
            hline = Span(location=price, 
                         dimension='width',
                         line_color='black',
                         line_dash='dashed',
                         line_width=linewidth)
            linked_fig.add_layout(hline)
    
    ''' ----------------------- TOP FIG PLOTTING -------------------------- '''
    
    def _plot_trade(self, x_data, y_data, marker_type, marker_colour, 
                    label, linked_fig, scatter_size=15):
        '''
        Plots individual trade.
        '''
        
        linked_fig.scatter(x_data, y_data,
                           marker       = marker_type,
                           size         = scatter_size,
                           fill_color   = marker_colour,
                           legend_label = label)
    
    def _plot_trade_history(self, trade_summary, linked_fig, 
                            cancelled_summary=False, open_summary=False):
        ''' Plots trades taken over ohlc chart. '''
        
        ts = trade_summary
        # TODO - merge should work with left_on='date', right_index=True,
        # meaning this can be deleted below - test it
        ts['date']   = ts.index 
        ts           = ts.reset_index(drop = True)
        
        trade_summary = pd.merge(self.data, ts, left_on='date', right_on='date')
        
        # Backtesting signals
        long_trades             = trade_summary[trade_summary.Size > 0]
        shorts_trades           = trade_summary[trade_summary.Size < 0]
        
        if cancelled_summary is False and open_summary is False:
            
            exit_summary = pd.merge(self.data, ts, left_on='date', right_on='Exit_time')
            
            profitable_longs        = long_trades[(long_trades['Profit'] > 0)]
            unprofitable_longs      = long_trades[(long_trades['Profit'] < 0)]
            profitable_shorts       = shorts_trades[(shorts_trades['Profit'] > 0)]
            unprofitable_shorts     = shorts_trades[(shorts_trades['Profit'] < 0)]
            
            # Profitable long trades
            if len(profitable_longs) > 0:
                self._plot_trade(list(profitable_longs.data_index.values),
                                 list(profitable_longs.Entry.values), 
                                 'triangle', 'lightgreen', 
                                 'Profitable long trades', linked_fig)
    
            # Profitable short trades
            if len(profitable_shorts) > 0:
                self._plot_trade(list(profitable_shorts.data_index.values),
                                 list(profitable_shorts.Entry.values),
                                 'inverted_triangle', 'lightgreen',
                                 'Profitable short trades', linked_fig)
            
            # Unprofitable long trades
            if len(unprofitable_longs) > 0:
                self._plot_trade(list(unprofitable_longs.data_index.values),
                                 list(unprofitable_longs.Entry.values),
                                 'triangle', 'orangered',
                                 'Unprofitable long trades', linked_fig)
            
            # Unprofitable short trades
            if len(unprofitable_shorts) > 0:
                self._plot_trade(list(unprofitable_shorts.data_index.values),
                                 list(unprofitable_shorts.Entry.values),
                                 'inverted_triangle', 'orangered',
                                 'Unprofitable short trades', linked_fig)
        else:
            if cancelled_summary:
                long_legend_label = 'Cancelled long trades'
                short_legend_label = 'Cancelled short trades'
                fill_color = 'black'
                price = 'Order_price'
            else:
                long_legend_label = 'Open long trades'
                short_legend_label = 'Open short trades'
                fill_color = 'white'
                price = 'Entry'
        
            # Partial long trades
            if len(long_trades) > 0:
                linked_fig.scatter(list(long_trades.data_index.values),
                                   list(long_trades[price].values),
                                   marker = 'triangle',
                                   size = 15,
                                   fill_color = fill_color,
                                   legend_label = long_legend_label)
            
            # Partial short trades
            if len(shorts_trades) > 0:
                linked_fig.scatter(list(shorts_trades.data_index.values),
                                   list(shorts_trades[price].values),
                                   marker = 'inverted_triangle',
                                   size = 15,
                                   fill_color = fill_color,
                                   legend_label = short_legend_label)
        
        
        # Stop loss  levels
        if None not in trade_summary.Stop_loss.values:
            self._plot_trade(list(trade_summary.data_index.values),
                             list(trade_summary.Stop_loss.values),
                             'dash', 'black', 'Stop loss', linked_fig)
        
        # Take profit levels
        if None not in trade_summary.Take_profit.values:
            self._plot_trade(list(trade_summary.data_index.values),
                             list(trade_summary.Take_profit.values),
                             'dash', 'black', 'Take profit', linked_fig)
        
        # Position exits
        if cancelled_summary is False and open_summary is False:
            self._plot_trade(list(exit_summary.data_index),
                             list(exit_summary.Exit_price.values),
                             'circle', 'black', 'Position exit', linked_fig,
                             scatter_size=7)
    
    
    ''' --------------------- BOTTOM FIG PLOTTING ------------------------- '''
    
    def _plot_macd(self, x_range, macd_data, linked_fig):
        ''' Plots MACD indicator. '''
        # Initialise figure
        fig = figure(plot_width     = linked_fig.plot_width,
                     plot_height    = self.bottom_fig_height,
                     title          = None,
                     tools          = linked_fig.tools,
                     active_drag    = linked_fig.tools[0],
                     active_scroll  = linked_fig.tools[1],
                     x_range        = linked_fig.x_range)
        
        # Add glyphs
        fig.line(x_range, macd_data['macd'], line_color = 'blue')
        fig.line(x_range, macd_data['signal'], line_color = 'red')
        if 'histogram' in macd_data:
            histcolour = []
            for i in range(len(macd_data['histogram'])):
                if np.isnan(macd_data['histogram'][i]):
                    histcolour.append('lightblue')
                else:
                    if macd_data['histogram'][i] < 0:
                        histcolour.append('red')
                    else:
                        histcolour.append('lightblue')
                        
            fig.quad(top = macd_data['histogram'],
                     bottom = 0,
                     left = x_range - 0.3,
                     right = x_range + 0.3,
                     fill_color = histcolour)
            
        if 'crossvals' in macd_data:
            fig.scatter(x_range,
                        macd_data['crossvals'],
                        marker = 'dash',
                        size = 15,
                        fill_color = 'black',
                        legend_label = 'Last Crossover Value')
    
        return fig
    
    ''' -------------------- MISCELLANEOUS PLOTTING ----------------------- '''
    def _plot_bars(self, x_vals, data_name, source, linked_fig=None, fig_height=250,
                   fig_title=None, hover_name=None):
        fig = figure(x_range = x_vals,
                     title = fig_title,
                     toolbar_location = None,
                     tools = 'hover',
                     tooltips = "@index: @{}".format(hover_name),
                     plot_height = fig_height)
        
        fig.vbar(x = 'index', 
                 top = data_name,
                 width = 0.9,
                 color = 'color',
                 source = source)
        
        return fig
    
    def _plot_pie(self, source, fig_title=None, fig_height=250):
        
        pie = figure(title = fig_title, 
                     toolbar_location = None,
                     tools = "hover", 
                     tooltips="@instrument: @value",
                     x_range=(-1, 1),
                     y_range=(0.0, 2.0),
                     plot_height = fig_height)
        
        pie.wedge(x=0, y=1, radius=0.3,
                  start_angle=cumsum('angle', include_zero=True), 
                  end_angle=cumsum('angle'),
                  line_color="white", 
                  fill_color='color',
                  legend_field='instrument',
                  source=source)
        
        return pie
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Logging script. Writes log file.
"""
from datetime import datetime
import glob
import os  
import pyfiglet

def write_backtest_log(pair, config, trade_summary):
    ''' 
    Function to write backtest results to a log file. 
    '''
    # Banner
    banner = pyfiglet.figlet_format("AutoBacktest")

    # Config file
    interval            = config["STRATEGY"]["INTERVAL"]
    period              = config["STRATEGY"]["PERIOD"]
    params              = config["STRATEGY"]["PARAMETERS"]
    risk_pc             = config["STRATEGY"]["RISK_PC"]
    strat_module        = config["STRATEGY"]["MODULE"]
    strat_name          = config["STRATEGY"]["NAME"]
    exit_function       = config["STRATEGY"]["EXIT_FUNCTION"]
    
    from_year           = config["BACKTESTING"]["FROM"]["year"]
    from_month          = config["BACKTESTING"]["FROM"]["month"]
    from_day            = config["BACKTESTING"]["FROM"]["day"]
    to_year             = config["BACKTESTING"]["TO"]["year"]
    to_month            = config["BACKTESTING"]["TO"]["month"]
    to_day              = config["BACKTESTING"]["TO"]["day"]
    
    from_date           = str(from_day) + "/" + str(from_month) + "/" + str(from_year)
    to_date             = str(to_day) + "/" + str(to_month) + "/" + str(to_year)
    
    # Trade summary
    no_trades           = len(trade_summary)
    final_balance       = trade_summary.Balance[-1]
    initial_balance     = trade_summary.Balance[0] - trade_summary.Profit[0]
    profit              = final_balance - initial_balance
    profit_pc           = 100*profit / initial_balance
    profitable_longs        = trade_summary[(trade_summary['Profit'] > 0) 
                                            & (trade_summary['Size'] > 0)]
    profitable_shorts       = trade_summary[(trade_summary['Profit'] > 0) 
                                            & (trade_summary['Size'] < 0)]
    no_profitable_trades    = len(profitable_longs) + len(profitable_shorts)
    win_rate                = 100*no_profitable_trades/no_trades
    
    time            = datetime.now().strftime("%H:%M:%S")
    
    # Create file name
    file_dir        = os.path.dirname(os.path.abspath(__file__))
    log_dir         = os.path.join(file_dir, '..', 'logfiles')
    log_file_dir    = os.path.join(log_dir, '*{}*'.format(strat_module))
    log_files       = glob.glob(log_file_dir)
    if len(log_files) != 0:
        file_no = len(log_files) + 1
    else:
        file_no = 1
    
    outFileName     = strat_module + "_log_" + str(file_no) + ".txt"
    outFileName_abs = os.path.join(log_dir, outFileName)
    
    with open(outFileName_abs, 'w+') as f:
        f.write(banner)
        f.write("Log file created by AutoTrader at {}\n".format(time))
        f.write("-------------------------------------------\n")
        f.write("Strategy: {}\n".format(strat_name))
        f.write("Backtest period: {0} to {1}\n".format(from_date, to_date))
        f.write("\nResults\n")
        f.write("-------\n")
        f.write("  Number of trades: {}\n".format(no_trades))
        f.write("  Starting balance: ${}\n".format(round(initial_balance, 2)))
        f.write("  Final balance: ${}\n".format(round(final_balance, 2)))
        f.write("  Profit: {0} ({1}%)\n".format(round(profit, 2), 
                                                round(profit_pc, 1)))
        f.write("  Win rate: {}%\n".format(round(win_rate, 3)))
        f.write("\n")
        f.write("The configuration file used in this strategy is provided below.\n")
        f.write("\n")
        f.write("STRATEGY:\n")
        f.write("  MODULE: '{}'\n".format(strat_module))
        f.write("  NAME: '{}'\n".format(strat_name))
        f.write("  INTERVAL: '{}'\n".format(interval))
        f.write("  PERIOD: {}\n".format(period))
        f.write("  RISK_PC: {}\n".format(risk_pc))
        f.write("  EXIT_FUNCTION: {}\n".format(exit_function))
        
        f.write("  PARAMETERS: \n")
        for key in params:
            param_str = "    " + key + ": {}\n".format(params[key])
            f.write(param_str)
        f.write("\n")
        
        f.write("BROKER: \n")
        f.write("  initial_balance: {}\n".format(int(initial_balance)))
        f.write("\n")
        
        f.write("WATCHLIST: ['" + pair + "']\n\n")
        
        f.write("BACKTESTING:\n")
        f.write("  FROM:\n")
        f.write("    year: {}\n".format(from_year))
        f.write("    month: {}\n".format(from_month))
        f.write("    day: {}\n".format(from_day))
        f.write("  TO:\n")
        f.write("    year: {}\n".format(to_year))
        f.write("    month: {}\n".format(to_month))
        f.write("    day: {}\n".format(to_day))
        f.write("\n")
    




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='autotrader',
    version='0.4.25',
    author='Kieran Mackle',
    author_email='kemackle98@gmail.com',
    description="A Python-based platform for developing, optimising and deploying automated trading systems.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/kieran-mackle/AutoTrader',
    project_urls={
        "Bug Tracker": "https://github.com/kieran-mackle/AutoTrader/issues",
        "Website": "https://kieran-mackle.github.io/AutoTrader/",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "."},
    packages=setuptools.find_packages(where="."),
    python_requires=">=3.6",
    install_requires = [
        "pandas",
        "pyfiglet",
        "PyYAML",
        "bokeh >= 2.3.1",
        "scipy",
        "yfinance",
        "finta"
        ],
    setup_requires=[
            'setuptools_git',
            'setuptools_scm',
        ],
    package_data={'': ['*.js']},
    include_package_data=True,
)
